// mcts.cpp

#include "node.h"

using namespace std::chrono;

umove searchMC(State *rootstate, int niterations, milliseconds timelimit)
{
    umove mv;
    mv.u = 0;
    Node* rootnode = new Node(mv, nullptr, rootstate);

    int iterations = 0;

    system_clock::time_point old = system_clock::now();

    while (duration_cast<milliseconds>(system_clock::now() - old) < timelimit || iterations < niterations)
    {
        iterations++;

        Node* node = rootnode;
        State* state = new State(*rootstate);

        // Select
        if (node->untriedMoves.empty() && !node->childNodes.empty())
        {
            node = node->selectChild();
            state->makemove(node->mv.b);
        }

        // Expand
        if (!node->untriedMoves.empty())
        {
            umove m = node->untriedMoves[0];
            state->makemove(m.b);
            node = node->addChild(m,state);
        }

        // Simulation or Scoring (maxdepth = 3)
        int g = -state->simulation();

        // backpropagate
        while (node != nullptr)
        {
            node->update(g);
            g = -g;
            node = node->parentNode;
        }

        delete state;
    }

//    std::cout << "iter: " << iterations << std::endl;

    double score = 0;
    umove bestMove;

    bool flag = true;

    for (auto& n : rootnode->childNodes)
    {
        umove m = n->mv;
        double sc = n->visits;

        if (flag)
        {
            score = sc;
            bestMove = m;
            flag = false;
        }
//        n->print(m, n->reward, n->visits, n->reward/n->visits, n->mean);

        if (sc > score)
        {
            score = sc;
            bestMove = m;
        }
    }

//    std::cout << std::endl;

    delete rootnode;

    return bestMove;
}
