#include "node.h"
#include <math.h>

Node::Node(umove m, Node* parent, State* state)
    : mv(m)
    , visits(0)
    , mean(0)
    , alpha(-MAXSCORE)
    , beta(MAXSCORE)
    , maxNode(true)
    , parentNode(parent)
{
    if (parent != nullptr)
    {
        beta = MIN(beta, parent->beta);
    }

    state->genLegal(untriedMoves, alpha, beta); // see utility.cpp
}

Node *Node::selectChild()
{
    assert(childNodes.size() > 0);

    Node* selected = nullptr;

    double bestValue = INT_MIN;

    for (auto& n : childNodes)
    {
        assert(visits > 0);

        double utc = n->mean + 1.4 * sqrt(log(visits)/n->visits);

        if (utc > bestValue)
        {
            selected = n;
            bestValue = utc;
        }
    }

    assert(selected != nullptr);

    return selected;
}

Node *Node::addChild(umove m, State *state)
{
    Node* n = new Node(m, this, state);

     for (auto it = untriedMoves.begin(); it != untriedMoves.end(); )
     {
         umove um = *it;

         if (um.u == m.u)
         {
             untriedMoves.erase(it);
             break;
         } 
		 else 
		 {
             ++it;
         }
     }

     childNodes.push_back(n);

     return n;
}

void Node::update(int result)
{
    visits++;
    mean += result/visits;

    if (maxNode)
    {
        alpha = MAX(alpha, result);
        beta  = MAX(beta, result);
    }
    else
    {
        alpha = MIN(alpha, result);
        beta  = MIN(beta, result);
    }
    maxNode = !maxNode;
}

Node::~Node()
{
    for (auto &n : childNodes)
    {
        delete n;
    }
}
